@@ -65,6 +65,7 @@ static struct history cmd_history;
 static char *cmd_history_filename;
 static char *history_search_text = NULL;
 static int arg_expand_cmd = -1;
+static int prev_view = -1;
 
 static char *get_home_dir(const char *username)
 {
@@ -268,15 +269,23 @@ void view_save(int view, char *arg)
 	switch (view) {
 	case TREE_VIEW:
 	case SORTED_VIEW:
+		if (worker_has_job(JOB_TYPE_LIB))
+			goto worker_running;
 		do_save(lib_for_each, arg, &lib_filename);
 		break;
 	case PLAYLIST_VIEW:
+		if (worker_has_job(JOB_TYPE_PL))
+			goto worker_running;
 		do_save(pl_for_each, arg, &pl_filename);
 		break;
 	default:
 		info_msg(":save only works in views 1 & 2 (library) and 3 (playlist)");
 	}
 	free(arg);
+	return;
+worker_running:
+	error_msg("can't save when tracks are being added");
+	free(arg);
 }
 
 /* }}} */
@@ -688,7 +697,8 @@ err:
 
 static void cmd_quit(char *arg)
 {
-	quit();
+	if (!worker_has_job(JOB_TYPE_ANY) || yes_no_query("Tracks are being added. Quit and truncate playlist(s)? [y/N]"))
+		cmus_running = 0;
 }
 
 static void cmd_reshuffle(char *arg)
@@ -1181,12 +1191,24 @@ err:
 	error_msg("expecting 1 or 2 arguments (total or L and R volumes [+-]INTEGER[%%])\n");
 }
 
+static void cmd_prev_view(char *arg)
+{
+	int tmp;
+	if (prev_view >= 0) {
+		tmp = cur_view;
+		set_view(prev_view);
+		prev_view = tmp;
+	}
+}
+
 static void cmd_view(char *arg)
 {
 	int view;
 
-	if (parse_enum(arg, 1, NR_VIEWS, view_names, &view))
+	if (parse_enum(arg, 1, NR_VIEWS, view_names, &view) && (view - 1) != cur_view) {
+		prev_view = cur_view;
 		set_view(view - 1);
+	}
 }
 
 static void cmd_p_next(char *arg)
@@ -1202,7 +1224,7 @@ static void cmd_p_pause(char *arg)
 static void cmd_p_play(char *arg)
 {
 	if (arg) {
-		player_play_file(arg);
+		cmus_play_file(arg);
 	} else {
 		player_play();
 	}
@@ -1376,8 +1398,7 @@ static void cmd_win_activate(char *arg)
 		if (cur_view == 2)
 			play_library = 0;
 
-		player_play_file(info->filename);
-		track_info_unref(info);
+		player_play_file(info);
 	}
 }
 
@@ -1615,6 +1636,197 @@ static void cmd_refresh(char *arg)
 	refresh();
 }
 
+static int cmp_intp(const void *ap, const void *bp)
+{
+	int a = *(int *)ap;
+	int b = *(int *)bp;
+	return a - b;
+}
+
+static int *rand_array(int size, int nmax)
+{
+	int *r = xnew(int, size + 1);
+	int i, offset = 0;
+	int count = size;
+
+	if (count > nmax / 2) {
+		/*
+		 * Imagine that there are 1000 tracks in library and we want to
+		 * add 998 random tracks to queue.  After we have added 997
+		 * random numbers to the array it would be quite hard to find a
+		 * random number that isn't already in the array (3/1000
+		 * probability).
+		 *
+		 * So we invert the logic:
+		 *
+		 * Find two (1000 - 998) random numbers in 0..999 range and put
+		 * them at end of the array.  Sort the numbers and then fill
+		 * the array starting at index 0 with incrementing values that
+		 * are not in the set of random numbers.
+		 */
+		count = nmax - count;
+		offset = size - count;
+	}
+
+	for (i = 0; i < count; ) {
+		int v, j;
+found:
+		v = rand() % nmax;
+		for (j = 0; j < i; j++) {
+			if (r[offset + j] == v)
+				goto found;
+		}
+		r[offset + i++] = v;
+	}
+	qsort(r + offset, count, sizeof(*r), cmp_intp);
+
+	if (offset) {
+		int j, n;
+
+		/* simplifies next loop */
+		r[size] = nmax;
+
+		/* convert the indexes we don't want to those we want */
+		i = 0;
+		j = offset;
+		n = 0;
+		do {
+			while (n < r[j])
+				r[i++] = n++;
+			j++;
+			n++;
+		} while (i < size);
+	}
+	return r;
+}
+
+static int count_albums(void)
+{
+	struct artist *artist;
+	struct list_head *item;
+	int count = 0;
+
+	list_for_each_entry(artist, &lib_artist_head, node) {
+		list_for_each(item, &artist->album_head)
+			count++;
+	}
+	return count;
+}
+
+struct album_list {
+	struct list_head node;
+	const struct album *album;
+};
+
+static void cmd_lqueue(char *arg)
+{
+	LIST_HEAD(head);
+	const struct list_head *item;
+	const struct album *album;
+	int count = 1, nmax, i, pos;
+	int *r;
+
+	if (arg) {
+		long int val;
+
+		if (str_to_int(arg, &val) || val <= 0) {
+			error_msg("argument must be positive integer");
+			return;
+		}
+		count = val;
+	}
+	editable_lock();
+	nmax = count_albums();
+	if (count > nmax)
+		count = nmax;
+	if (!count)
+		goto unlock;
+
+	r = rand_array(count, nmax);
+	album = to_album(to_artist(lib_artist_head.next)->album_head.next);
+	pos = 0;
+	for (i = 0; i < count; i++) {
+		struct album_list *a;
+
+		while (pos < r[i]) {
+			struct artist *artist = album->artist;
+			if (album->node.next == &artist->album_head) {
+				artist = to_artist(artist->node.next);
+				album = to_album(artist->album_head.next);
+			} else {
+				album = to_album(album->node.next);
+			}
+			pos++;
+		}
+		a = xnew(struct album_list, 1);
+		a->album = album;
+		list_add_rand(&head, &a->node, i);
+	}
+	free(r);
+
+	item = head.next;
+	do {
+		struct list_head *next = item->next;
+		struct album_list *a = container_of(item, struct album_list, node);
+		struct tree_track *t;
+
+		list_for_each_entry(t, &a->album->track_head, node)
+			editable_add(&pq_editable, simple_track_new(tree_track_info(t)));
+		free(a);
+		item = next;
+	} while (item != &head);
+unlock:
+	editable_unlock();
+}
+
+static void cmd_tqueue(char *arg)
+{
+	LIST_HEAD(head);
+	struct list_head *item;
+	int count = 1, i, pos;
+	int *r;
+
+	if (arg) {
+		long int val;
+
+		if (str_to_int(arg, &val) || val <= 0) {
+			error_msg("argument must be positive integer");
+			return;
+		}
+		count = val;
+	}
+	editable_lock();
+	if (count > lib_editable.nr_tracks)
+		count = lib_editable.nr_tracks;
+	if (!count)
+		goto unlock;
+
+	r = rand_array(count, lib_editable.nr_tracks);
+	item = lib_editable.head.next;
+	pos = 0;
+	for (i = 0; i < count; i++) {
+		struct simple_track *t;
+
+		while (pos < r[i]) {
+			item = item->next;
+			pos++;
+		}
+		t = simple_track_new(to_simple_track(item)->info);
+		list_add_rand(&head, &t->node, i);
+	}
+	free(r);
+
+	item = head.next;
+	do {
+		struct list_head *next = item->next;
+		struct simple_track *t = to_simple_track(item);
+		editable_add(&pq_editable, t);
+		item = next;
+	} while (item != &head);
+unlock:
+	editable_unlock();
+}
+
 /* tab exp {{{
  *
  * these functions fill tabexp struct, which is resetted beforehand
@@ -1736,7 +1948,6 @@ static void expand_key_context(const char *str, const char *force)
 	snprintf(expbuf, sizeof(expbuf), "%s%s", force, str);
 	tabexp.head = xstrdup(expbuf);
 	tabexp.tails = tails;
-	tabexp.nr_tails = pos;
 }
 
 static int get_context(const char *str, int len)
@@ -1832,7 +2043,6 @@ static void expand_bind_args(const char *str)
 		ptr_array_plug(&array);
 		tabexp.head = xstrdup(expbuf);
 		tabexp.tails = array.ptrs;
-		tabexp.nr_tails = array.count;
 		return;
 	}
 
@@ -1926,7 +2136,6 @@ static void expand_unbind_args(const char *str)
 	ptr_array_plug(&array);
 	tabexp.head = xstrdup(expbuf);
 	tabexp.tails = array.ptrs;
-	tabexp.nr_tails = array.count;
 }
 
 static void expand_factivate(const char *str)
@@ -1957,7 +2166,6 @@ static void expand_factivate(const char *str)
 	ptr_array_plug(&array);
 	tabexp.head = xstrdup(str);
 	tabexp.tails = array.ptrs;
-	tabexp.nr_tails = array.count;
 }
 
 static void expand_options(const char *str)
@@ -1985,7 +2193,6 @@ static void expand_options(const char *str)
 
 				tabexp.head = xstrdup(str);
 				tabexp.tails = tails;
-				tabexp.nr_tails = 1;
 				free(var);
 				return;
 			}
@@ -2013,7 +2220,6 @@ static void expand_options(const char *str)
 			tails[pos] = NULL;
 			tabexp.head = xstrdup(str);
 			tabexp.tails = tails;
-			tabexp.nr_tails = pos;
 		} else {
 			free(tails);
 		}
@@ -2039,7 +2245,6 @@ static void expand_toptions(const char *str)
 		tails[pos] = NULL;
 		tabexp.head = xstrdup(str);
 		tabexp.tails = tails;
-		tabexp.nr_tails = pos;
 	} else {
 		free(tails);
 	}
@@ -2086,7 +2291,6 @@ static void expand_colorscheme(const char *str)
 		ptr_array_plug(&array);
 		tabexp.head = xstrdup(str);
 		tabexp.tails = array.ptrs;
-		tabexp.nr_tails = array.count;
 	}
 }
 
@@ -2106,12 +2310,14 @@ struct command commands[] = {
 	{ "fset",		cmd_fset,	1, 1, NULL,		  0, 0 },
 	{ "invert",		cmd_invert,	0, 0, NULL,		  0, 0 },
 	{ "load",		cmd_load,	1, 1, expand_load_save,	  0, 0 },
+	{ "lqueue",		cmd_lqueue,	0, 1, NULL,		  0, 0 },
 	{ "mark",		cmd_mark,	0, 1, NULL,		  0, 0 },
 	{ "player-next",	cmd_p_next,	0, 0, NULL,		  0, 0 },
 	{ "player-pause",	cmd_p_pause,	0, 0, NULL,		  0, 0 },
 	{ "player-play",	cmd_p_play,	0, 1, expand_playable,	  0, 0 },
 	{ "player-prev",	cmd_p_prev,	0, 0, NULL,		  0, 0 },
 	{ "player-stop",	cmd_p_stop,	0, 0, NULL,		  0, 0 },
+	{ "prev-view",  	cmd_prev_view,	0, 0, NULL,		  0, 0 },
 	{ "quit",		cmd_quit,	0, 0, NULL,		  0, 0 },
 	{ "refresh",		cmd_refresh,	0, 0, NULL,		  0, 0 },
 	{ "run",		cmd_run,	1,-1, NULL,		  0, CMD_UNSAFE },
@@ -2124,6 +2330,7 @@ struct command commands[] = {
 	{ "shuffle",		cmd_reshuffle,	0, 0, NULL,		  0, 0 },
 	{ "source",		cmd_source,	1, 1, expand_files,	  0, CMD_UNSAFE },
 	{ "toggle",		cmd_toggle,	1, 1, expand_toptions,	  0, 0 },
+	{ "tqueue",		cmd_tqueue,	0, 1, NULL,		  0, 0 },
 	{ "unbind",		cmd_unbind,	1, 1, expand_unbind_args, 0, 0 },
 	{ "unmark",		cmd_unmark,	0, 0, NULL,		  0, 0 },
 	{ "view",		cmd_view,	1, 1, NULL,		  0, 0 },
@@ -2174,7 +2381,6 @@ static void expand_commands(const char *str)
 		tails[pos] = NULL;
 		tabexp.head = xstrdup(str);
 		tabexp.tails = tails;
-		tabexp.nr_tails = pos;
 	} else {
 		free(tails);
 	}
@@ -2436,9 +2642,6 @@ void command_mode_ch(uchar ch)
 		cmdline_clear_end();
 		break;
 	case 0x09:
-		/* tab expansion should not complain  */
-		display_errors = 0;
-
 		tab_expand();
 		break;
 	case 0x15:
