@@ -20,7 +20,30 @@
 #ifndef _COMMAND_MODE_H
 #define _COMMAND_MODE_H
 
-#include <uchar.h>
+#include "uchar.h"
+
+/* executing command is disabled over net */
+#define CMD_UNSAFE 0x01
+
+struct command {
+	const char *name;
+	void (*func)(char *arg);
+
+	/* min/max number of arguments */
+	int min_args;
+	int max_args;
+
+	void (*expand)(const char *str);
+
+	/* bind count (0 means: unbound) */
+	int bc;
+
+	/* CMD_* */
+	unsigned int flags;
+};
+
+extern struct command commands[];
+extern int run_only_safe_commands;
 
 void command_mode_ch(uchar ch);
 void command_mode_key(int key);
@@ -28,6 +51,8 @@ void commands_init(void);
 void commands_exit(void);
 void run_command(const char *buf);
 
+struct command *get_command(const char *str);
+
 void view_clear(int view);
 void view_add(int view, char *arg, int prepend);
 void view_load(int view, char *arg);
