@@ -26,14 +26,6 @@
  * if v2 is at beginning _and_ at end then there must be a seek tag at beginning
  */
 
-struct ID3 {
-	char v1[128];
-	char *v2[NUM_ID3_KEYS];
-
-	unsigned int has_v1 : 1;
-	unsigned int has_v2 : 1;
-};
-
 struct v2_header {
 	unsigned char ver_major;
 	unsigned char ver_minor;
@@ -221,6 +213,23 @@ static const char *genres[NR_GENRES] = {
 #define id3_debug(...) do { } while (0)
 #endif
 
+const char * const id3_key_names[NUM_ID3_KEYS] = {
+	"artist",
+	"album",
+	"title",
+	"date",
+	"genre",
+	"discnumber",
+	"tracknumber",
+	"albumartist",
+	"artistsort",
+	"albumartistsort",
+	"replaygain_track_gain",
+	"replaygain_track_peak",
+	"replaygain_album_gain",
+	"replaygain_album_peak"
+};
+
 static int utf16_is_special(const uchar uch)
 {
 	if (UTF16_IS_HSURROGATE(uch) || UTF16_IS_LSURROGATE(uch) || UTF16_IS_BOM(uch))
@@ -505,6 +514,7 @@ static struct {
 	{ "TDRC", ID3_DATE }, // recording date
 	{ "TDRL", ID3_DATE }, // release date
 	{ "TDOR", ID3_DATE }, // original release date
+	{ "TSOP", ID3_ARTISTSORT },
 
 	/* >= 2.3.0 */
 	{ "TPE1", ID3_ARTIST },
@@ -515,6 +525,7 @@ static struct {
 	{ "TPOS", ID3_DISC },
 	{ "TRCK", ID3_TRACK },
 	{ "TPE2", ID3_ALBUMARTIST },
+	{ "XSOP", ID3_ARTISTSORT }, // obsolete
 
 	/* obsolete frames (2.2.0) */
 	{ "TP1",  ID3_ARTIST },
@@ -528,6 +539,17 @@ static struct {
 	{ "", -1 }
 };
 
+static int frame_tab_index(const char *id)
+{
+	int i;
+
+	for (i = 0; frame_tab[i].key != -1; i++) {
+		if (!strncmp(id, frame_tab[i].name, 4))
+			return i;
+	}
+	return -1;
+}
+
 static void fix_date(char *buf)
 {
 	const char *ptr = buf;
@@ -550,54 +572,52 @@ static void fix_date(char *buf)
 	*buf = 0;
 }
 
-static void v2_add_frame(ID3 *id3, struct v2_frame_header *fh, const char *buf)
+static char *decode_str(const char *buf, int len, int encoding)
 {
-	int i, encoding = *buf++, len = fh->size - 1;
+	char *in, *out = NULL;
+	int rc = 0;
+
+	switch (encoding) {
+	case 0x00: /* ISO-8859-1 */
+		in = xstrndup(buf, len);
+		rc = utf8_encode(in, id3_default_charset, &out);
+		free(in);
+		break;
+	case 0x03: /* UTF-8 */
+		in = xstrndup(buf, len);
+		if (u_is_valid(in)) {
+			out = in;
+		} else {
+			rc = utf8_encode(in, id3_default_charset, &out);
+			free(in);
+		}
+		break;
+	case 0x01: /* UTF-16 */
+		out = utf16_to_utf8((const unsigned char *)buf, len);
+		break;
+	case 0x02: /* UTF-16BE */
+		out = utf16be_to_utf8((const unsigned char *)buf, len);
+		break;
+	}
+	return out;
+}
+
+static void v2_add_frame(struct id3tag *id3, struct v2_frame_header *fh, const char *buf)
+{
+	int idx, encoding = *buf++, len = fh->size - 1;
+	enum id3_key key = NUM_ID3_KEYS;
+	char *out;
 
 	if (encoding > 3)
 		return;
 
-	for (i = 0; frame_tab[i].key != -1; i++) {
-		enum id3_key key = frame_tab[i].key;
-		char *in, *out;
-		int rc;
+	idx = frame_tab_index(fh->id);
+	if (idx >= 0) {
+		key = frame_tab[idx].key;
+		out = decode_str(buf, len, encoding);
+		if (!out)
+			return;
 
-		if (strncmp(fh->id, frame_tab[i].name, 4))
-			continue;
-
-		switch (encoding) {
-		case 0x00: /* ISO-8859-1 */
-			in = xstrndup(buf, len);
-			rc = utf8_encode(in, id3_default_charset, &out);
-			free(in);
-			if (rc)
-				return;
-			break;
-		case 0x03: /* UTF-8 */
-			in = xstrndup(buf, len);
-			if (u_is_valid(in)) {
-				out = in;
-			} else {
-				rc = utf8_encode(in, id3_default_charset, &out);
-				free(in);
-				if (rc)
-					return;
-			}
-			break;
-		case 0x01: /* UTF-16 */
-			out = utf16_to_utf8((const unsigned char *)buf, len);
-			if (out == NULL)
-				return;
-			break;
-		case 0x02: /* UTF-16BE */
-			out = utf16be_to_utf8((const unsigned char *)buf, len);
-			if (out == NULL)
-				return;
-			break;
-		}
-
-		if (key == ID3_TRACK || key == ID3_DISC)
-			fix_track_or_disc(out);
 		if (key == ID3_GENRE) {
 			char *tmp;
 
@@ -616,12 +636,54 @@ static void v2_add_frame(ID3 *id3, struct v2_frame_header *fh, const char *buf)
 			}
 		}
 
-		free(id3->v2[key]);
-		id3->v2[key] = out;
-		id3->has_v2 = 1;
-		id3_debug("%s '%s'\n", frame_tab[i].name, out);
-		break;
+		id3_debug("%s '%s'\n", frame_tab[idx].name, out);
+	} else if (!strncmp(fh->id, "TXXX", 4)) {
+		int size;
+
+		id3_debug("TXXX\n");
+
+		/* TXXX<len><encoding><key><val> */
+		out = decode_str(buf, len, encoding);
+		if (!out)
+			return;
+
+		id3_debug("TXXX, key = '%s'\n", out);
+		if (!strcasecmp(out, "replaygain_track_gain"))
+			key = ID3_RG_TRACK_GAIN;
+		if (!strcasecmp(out, "replaygain_track_peak"))
+			key = ID3_RG_TRACK_PEAK;
+		if (!strcasecmp(out, "replaygain_album_gain"))
+			key = ID3_RG_ALBUM_GAIN;
+		if (!strcasecmp(out, "replaygain_album_peak"))
+			key = ID3_RG_ALBUM_PEAK;
+		if (!strcasecmp(out, "album artist"))
+			key = ID3_ALBUMARTIST;
+		if (!strcasecmp(out, "albumartistsort"))
+			key = ID3_ALBUMARTISTSORT;
+
+		size = strlen(out) + 1;
+		free(out);
+
+		if (key == NUM_ID3_KEYS)
+			return;
+
+		buf += size;
+		len -= size;
+		if (len <= 0)
+			return;
+
+		out = decode_str(buf, len, encoding);
+		if (!out)
+			return;
+
+		id3_debug("TXXX, val = '%s'\n", out);
+	} else {
+		return;
 	}
+
+	free(id3->v2[key]);
+	id3->v2[key] = out;
+	id3->has_v2 = 1;
 }
 
 static void unsync(unsigned char *buf, int *lenp)
@@ -652,7 +714,7 @@ static void unsync(unsigned char *buf, int *lenp)
 	*lenp = d;
 }
 
-static int v2_read(ID3 *id3, int fd, const struct v2_header *header)
+static int v2_read(struct id3tag *id3, int fd, const struct v2_header *header)
 {
 	char *buf;
 	int rc, buf_size;
@@ -753,21 +815,15 @@ int id3_tag_size(const char *buf, int buf_size)
 	return 0;
 }
 
-ID3 *id3_new(void)
-{
-	return xnew0(ID3, 1);
-}
-
-void id3_free(ID3 *id3)
+void id3_free(struct id3tag *id3)
 {
 	int i;
 
 	for (i = 0; i < NUM_ID3_KEYS; i++)
 		free(id3->v2[i]);
-	free(id3);
 }
 
-int id3_read_tags(ID3 *id3, int fd, unsigned int flags)
+int id3_read_tags(struct id3tag *id3, int fd, unsigned int flags)
 {
 	off_t off;
 	int rc;
@@ -857,7 +913,7 @@ static char *v1_get_str(const char *buf, int len)
 	return out;
 }
 
-char *id3_get_comment(ID3 *id3, enum id3_key key)
+char *id3_get_comment(struct id3tag *id3, enum id3_key key)
 {
 	if (id3->has_v2) {
 		if (id3->v2[key])
@@ -891,9 +947,7 @@ char *id3_get_comment(ID3 *id3, enum id3_key key)
 				snprintf(t, 4, "%d", ((unsigned char *)id3->v1)[126]);
 				return t;
 			}
-		case ID3_DISC:
-		case ID3_ALBUMARTIST:
-		case NUM_ID3_KEYS:
+		default:
 			return NULL;
 		}
 	}
