@@ -20,6 +20,7 @@
 #include "ip.h"
 #include "nomad.h"
 #include "id3.h"
+#include "ape.h"
 #include "xmalloc.h"
 #include "read_wrapper.h"
 #include "debug.h"
@@ -113,23 +114,13 @@ static int mad_seek(struct input_plugin_data *ip_data, double offset)
 	return nomad_time_seek(nomad, offset);
 }
 
-static void get_comment(struct keyval *c, int *iptr, ID3 *id3, enum id3_key key, const char *key_name)
-{
-	int i = *iptr;
-
-	c[i].val = id3_get_comment(id3, key);
-	if (c[i].val == NULL)
-		return;
-	c[i].key = xstrdup(key_name);
-	*iptr = i + 1;
-}
-
 static int mad_read_comments(struct input_plugin_data *ip_data,
 		struct keyval **comments)
 {
-	ID3 *id3;
-	struct keyval *c;
+	struct id3tag id3;
 	int fd, rc, save, i;
+	APETAG(ape);
+	GROWING_KEYVALS(c);
 
 	fd = open(ip_data->filename, O_RDONLY);
 	if (fd == -1) {
@@ -137,8 +128,8 @@ static int mad_read_comments(struct input_plugin_data *ip_data,
 	}
 	d_print("filename: %s\n", ip_data->filename);
 
-	id3 = id3_new();
-	rc = id3_read_tags(id3, fd, ID3_V1 | ID3_V2);
+	id3_init(&id3);
+	rc = id3_read_tags(&id3, fd, ID3_V1 | ID3_V2);
 	save = errno;
 	close(fd);
 	errno = save;
@@ -148,22 +139,37 @@ static int mad_read_comments(struct input_plugin_data *ip_data,
 			return -1;
 		}
 		d_print("corrupted tag?\n");
-		*comments = xnew0(struct keyval, 1);
-		return 0;
+		goto next;
 	}
 
-	c = xnew0(struct keyval, NUM_ID3_KEYS + 1);
-	i = 0;
-	get_comment(c, &i, id3, ID3_ARTIST, "artist");
-	get_comment(c, &i, id3, ID3_ALBUM, "album");
-	get_comment(c, &i, id3, ID3_TITLE, "title");
-	get_comment(c, &i, id3, ID3_DATE, "date");
-	get_comment(c, &i, id3, ID3_GENRE, "genre");
-	get_comment(c, &i, id3, ID3_DISC, "discnumber");
-	get_comment(c, &i, id3, ID3_TRACK, "tracknumber");
-	get_comment(c, &i, id3, ID3_ALBUMARTIST, "albumartist");
-	*comments = c;
-	id3_free(id3);
+	for (i = 0; i < NUM_ID3_KEYS; i++) {
+		char *val = id3_get_comment(&id3, i);
+
+		if (val)
+			comments_add(&c, id3_key_names[i], val);
+	}
+
+next:
+	id3_free(&id3);
+
+	rc = ape_read_tags(&ape, ip_data->fd, 0);
+	if (rc < 0)
+		goto out;
+
+	for (i = 0; i < rc; i++) {
+		char *k, *v;
+		k = ape_get_comment(&ape, &v);
+		if (!k)
+			break;
+		comments_add(&c, k, v);
+		free(k);
+	}
+
+out:
+	ape_free(&ape);
+
+	comments_terminate(&c);
+	*comments = c.comments;
 	return 0;
 }
 
@@ -186,5 +192,5 @@ const struct input_plugin_ops ip_ops = {
 
 const char * const ip_extensions[] = { "mp3", "mp2", NULL };
 const char * const ip_mime_types[] = {
-	"audio/mpeg", "audio/x-mp3", "audio/x-mpeg", "audio/x-mpegurl", NULL
+	"audio/mpeg", "audio/x-mp3", "audio/x-mpeg", NULL
 };
