@@ -1,5 +1,5 @@
-#include <mergesort.h>
-#include <list.h>
+#include "mergesort.h"
+#include "list.h"
 
 void list_mergesort(struct list_head *head,
 	int (*compare)(const struct list_head *, const struct list_head *))
