@@ -5,6 +5,7 @@
  */
 
 #include "ip.h"
+#include "ape.h"
 #include "comment.h"
 #include "file.h"
 #include "xmalloc.h"
@@ -14,35 +15,6 @@
 #include <inttypes.h>
 #include <errno.h>
 
-/* http://www.personal.uni-jena.de/~pfk/mpp/sv8/apetag.html */
-
-#define PREAMBLE_SIZE (8)
-static const char preamble[PREAMBLE_SIZE] = { 'A', 'P', 'E', 'T', 'A', 'G', 'E', 'X' };
-
-/* NOTE: not sizeof(struct ape_header)! */
-#define HEADER_SIZE (32)
-
-struct ape_header {
-	/* 1000 or 2000 (1.0, 2.0) */
-	uint32_t version;
-
-	/* tag size (header + tags, excluding footer) */
-	uint32_t size;
-
-	/* number of items */
-	uint32_t count;
-
-	/* global flags for each tag
-	 * there are also private flags for every tag
-	 * NOTE: 0 for version 1.0 (1000)
-	 */
-	uint32_t flags;
-};
-
-/* ape flags */
-#define AF_IS_UTF8(f)		(((f) & 6) == 0)
-#define AF_IS_FOOTER(f)		(((f) & (1 << 29)) == 0)
-
 struct mpc_private {
 	mpc_decoder decoder;
 	mpc_reader reader;
@@ -67,14 +39,7 @@ struct mpc_private {
 	MPC_SAMPLE_FORMAT samples[MPC_DECODER_BUFFER_LENGTH];
 };
 
-static inline uint32_t get_le32(const char *buf)
-{
-	const unsigned char *b = (const unsigned char *)buf;
-
-	return b[0] | (b[1] >> 8) | (b[2] >> 16) | (b[3] >> 24);
-}
-
-/* callbacks {{{ */
+/* callbacks */
 static mpc_int32_t read_impl(void *data, void *ptr, mpc_int32_t size)
 {
 	struct input_plugin_data *ip_data = data;
@@ -122,184 +87,6 @@ static mpc_bool_t canseek_impl(void *data)
 
 	return !ip_data->remote;
 }
-/* }}} */
-
-/* ape {{{ */
-
-/* returns position of APE header or -1 if not found */
-static int find_ape_tag_slow(int fd)
-{
-	char buf[4096];
-	int match = 0;
-	int pos = 0;
-
-	/* seek to start of file */
-	if (lseek(fd, pos, SEEK_SET) == -1)
-		return -1;
-
-	while (1) {
-		int i, got = read(fd, buf, sizeof(buf));
-
-		if (got == -1) {
-			if (errno == EAGAIN || errno == EINTR)
-				continue;
-			break;
-		}
-		if (got == 0)
-			break;
-
-		for (i = 0; i < got; i++) {
-			if (buf[i] != preamble[match]) {
-				match = 0;
-				continue;
-			}
-
-			match++;
-			if (match == PREAMBLE_SIZE)
-				return pos + i + 1 - PREAMBLE_SIZE;
-		}
-		pos += got;
-	}
-	return -1;
-}
-
-static int read_header(int fd, struct ape_header *h)
-{
-	char buf[HEADER_SIZE];
-
-	if (read_all(fd, buf, sizeof(buf)) != sizeof(buf))
-		return 0;
-	if (memcmp(buf, preamble, PREAMBLE_SIZE))
-		return 0;
-
-	h->version = get_le32(buf + 8);
-	h->size = get_le32(buf + 12);
-	h->count = get_le32(buf + 16);
-	h->flags = get_le32(buf + 20);
-	return 1;
-}
-
-/* sets fd right after the header and returns 1 if found,
- * otherwise returns 0
- */
-static int find_ape_tag(int fd, struct ape_header *h)
-{
-	int pos;
-
-	if (lseek(fd, -HEADER_SIZE, SEEK_END) == -1)
-		return 0;
-	if (read_header(fd, h))
-		return 1;
-
-	pos = find_ape_tag_slow(fd);
-	if (pos == -1)
-		return 0;
-	if (lseek(fd, pos, SEEK_SET) == -1)
-		return 0;
-	return read_header(fd, h);
-}
-
-/*
- * All keys are ASCII and length is 2..255
- *
- * UTF-8:	Artist, Album, Title, Genre
- * Integer:	Track (N or N/M)
- * Date:	Year (release), "Record Date"
- *
- * UTF-8 strings are NOT zero terminated.
- *
- * Also support "discnumber" (vorbis) and "disc" (non-standard)
- */
-static int parse_ape_tag(char *buf, const int size, const int count, struct keyval **comments)
-{
-	struct keyval *c;
-	int i, n, pos;
-
-	c = xnew0(struct keyval, count + 1);
-	i = 0;
-	pos = 0;
-	for (n = 0; n < count; n++) {
-		uint32_t val_len, flags;
-		char *key, *val;
-		int max_key_len, key_len;
-
-		val_len = get_le32(buf + pos); pos += 4;
-		flags = get_le32(buf + pos); pos += 4;
-
-		max_key_len = size - pos - val_len - 1;
-		if (max_key_len < 0) {
-			/* corrupt */
-			break;
-		}
-
-		for (key_len = 0; key_len < max_key_len && buf[pos + key_len]; key_len++)
-			; /* nothing */
-		if (buf[pos + key_len]) {
-			/* corrupt */
-			break;
-		}
-
-		if (!AF_IS_UTF8(flags)) {
-			/* ignore binary data */
-			pos += key_len + 1 + val_len;
-			continue;
-		}
-
-		key = xstrdup(buf + pos);
-		pos += key_len + 1;
-
-		/* should not be NUL-terminated */
-		val = xstrndup(buf + pos, val_len);
-		pos += val_len;
-
-		/* normalize key names */
-		if (!strcasecmp(key, "track")) {
-			free(key);
-			key = xstrdup("tracknumber");
-		} else if (!strcasecmp(key, "disc")) {
-			/* "disc" is totally non-standard but since there is
-			 * "track" to "tracknumber" conversion this might make sense,
-			 * at least this is totally harmless
-			 */
-			free(key);
-			key = xstrdup("discnumber");
-		}
-
-		if (!is_interesting_key(key)) {
-			free(key);
-			free(val);
-			continue;
-		}
-
-		if (!strcasecmp(key, "tracknumber") || !strcasecmp(key, "discnumber")) {
-			fix_track_or_disc(val);
-		} else if (!strcasecmp(key, "year") || !strcasecmp(key, "record date")) {
-			/* Date format
-			 *
-			 * 1999-08-11 12:34:56
-			 * 1999-08-11 12:34
-			 * 1999-08-11
-			 * 1999-08
-			 * 1999
-			 * 1999-W34	(week 34, totally crazy)
-			 *
-			 * convert to year, pl.c supports only years anyways
-			 *
-			 * FIXME: which one is the most common tag (year or record date)?
-			 */
-			if (strlen(val) > 4)
-				val[4] = 0;
-		}
-
-		c[i].key = key;
-		c[i].val = val;
-		i++;
-	}
-
-	*comments = c;
-	return 0;
-}
-/* }}} */
 
 static int mpc_open(struct input_plugin_data *ip_data)
 {
@@ -430,38 +217,60 @@ static int mpc_seek(struct input_plugin_data *ip_data, double offset)
 	return -1;
 }
 
+static const char *gain_to_str(int gain)
+{
+	static char buf[16];
+	int b, a = gain / 100;
+
+	if (gain < 0) {
+		b = -gain % 100;
+	} else {
+		b = gain % 100;
+	}
+	sprintf(buf, "%d.%02d", a, b);
+	return buf;
+}
+
+static const char *peak_to_str(unsigned int peak)
+{
+	static char buf[16];
+	sprintf(buf, "%d.%05d", peak / 32767, peak % 32767);
+	return buf;
+}
+
 static int mpc_read_comments(struct input_plugin_data *ip_data, struct keyval **comments)
 {
-	int old_pos, fd = ip_data->fd;
-	struct ape_header h;
-	char *buf;
+	struct mpc_private *priv = ip_data->private;
+	GROWING_KEYVALS(c);
+	int count, i;
+	APETAG(ape);
 
-	*comments = NULL;
-
-	/* save position */
-	old_pos = lseek(fd, 0, SEEK_CUR);
-
-	if (!find_ape_tag(fd, &h))
+	count = ape_read_tags(&ape, ip_data->fd, 1);
+	if (count < 0)
 		goto out;
 
-	if (AF_IS_FOOTER(h.flags)) {
-		/* seek back right after the header */
-		if (lseek(fd, -h.size, SEEK_CUR) == -1)
-			goto out;
+	for (i = 0; i < count; i++) {
+		char *k, *v;
+		k = ape_get_comment(&ape, &v);
+		if (!k)
+			break;
+		comments_add(&c, k, v);
+		free(k);
 	}
 
-	/* ignore insane tags */
-	if (h.size > 1024 * 1024)
-		goto out;
-
-	buf = xnew(char, h.size);
-	if (read_all(fd, buf, h.size) == h.size)
-		parse_ape_tag(buf, h.size, h.count, comments);
-	free(buf);
 out:
-	if (*comments == NULL)
-		*comments = xnew0(struct keyval, 1);
-	lseek(fd, old_pos, SEEK_SET);
+	if (priv->info.gain_title && priv->info.peak_title) {
+		comments_add_const(&c, "replaygain_track_gain", gain_to_str(priv->info.gain_title));
+		comments_add_const(&c, "replaygain_track_peak", peak_to_str(priv->info.peak_title));
+	}
+	if (priv->info.gain_album && priv->info.peak_album) {
+		comments_add_const(&c, "replaygain_album_gain", gain_to_str(priv->info.gain_album));
+		comments_add_const(&c, "replaygain_album_peak", peak_to_str(priv->info.peak_album));
+	}
+	comments_terminate(&c);
+
+	*comments = c.comments;
+	ape_free(&ape);
 	return 0;
 }
 
