@@ -41,7 +41,6 @@ struct nomad {
 	struct mad_stream stream;
 	struct mad_frame frame;
 	struct mad_synth synth;
-	struct mad_header header;
 	mad_timer_t timer;
 	unsigned long cur_frame;
 	off_t input_offset;
@@ -170,6 +169,10 @@ static int xing_parse(struct nomad *nomad)
 		bitlen -= 32;
 	}
 	nomad->has_xing = 1;
+#if defined(DEBUG_XING)
+	if (nomad->xing.flags & XING_FRAMES)
+		d_print("frames: %d (xing)\n", nomad->xing.nr_frames);
+#endif
 	return 0;
 }
 
@@ -241,23 +244,21 @@ static void handle_lost_sync(struct nomad *nomad)
 }
 
 
-/* builds a seek index as the file is decoded
- * should be called after every call to mad_decode_header,
- * and before increasing nomad->timer
+/* Builds a seek index as the file is decoded
+ * NOTE: increases nomad->timer (current position)
  */
 static void build_seek_index(struct nomad *nomad)
 {
-	mad_timer_t timer;
+	mad_timer_t timer_now = nomad->timer;
 	off_t offset;
 	int idx;
 
+	mad_timer_add(&nomad->timer, nomad->frame.header.duration);
+
 	if (nomad->has_xing)
 		return;
 
-	timer = nomad->timer;
-	mad_timer_add(&timer, nomad->header.duration);
-
-	if (timer.seconds < (nomad->seek_idx.size + 1) * SEEK_IDX_INTERVAL)
+	if (nomad->timer.seconds < (nomad->seek_idx.size + 1) * SEEK_IDX_INTERVAL)
 		return;
 
 	/* offset = ftell() */
@@ -271,11 +272,24 @@ static void build_seek_index(struct nomad *nomad)
 
 	nomad->seek_idx.table = xrenew(struct seek_idx_entry, nomad->seek_idx.table, idx + 1);
 	nomad->seek_idx.table[idx].offset = offset;
-	nomad->seek_idx.table[idx].timer = nomad->timer;
+	nomad->seek_idx.table[idx].timer = timer_now;
 
 	nomad->seek_idx.size++;
 }
 
+static void calc_fast(struct nomad *nomad)
+{
+	nomad->info.avg_bitrate = -1;
+	nomad->info.vbr = -1;
+	if (nomad->has_xing && (nomad->xing.flags & XING_FRAMES)) {
+		nomad->info.nr_frames = nomad->xing.nr_frames;
+		mad_timer_multiply(&nomad->timer, nomad->info.nr_frames);
+	} else {
+		nomad->info.nr_frames = nomad->info.filesize /
+			(nomad->stream.next_frame - nomad->stream.this_frame);
+		mad_timer_multiply(&nomad->timer, nomad->info.nr_frames);
+	}
+}
 
 /*
  * fields
@@ -285,7 +299,7 @@ static void build_seek_index(struct nomad *nomad)
  */
 static int scan(struct nomad *nomad)
 {
-	int frame_decoded = 0;
+	struct mad_header *header = &nomad->frame.header;
 	int old_bitrate = 0;
 	unsigned long long int bitrate_sum = 0;
 
@@ -300,64 +314,41 @@ static int scan(struct nomad *nomad)
 		if (rc == 0)
 			break;
 
-		if (mad_header_decode(&nomad->header, &nomad->stream) == 0) {
-			bitrate_sum += nomad->header.bitrate;
-			build_seek_index(nomad);
-			mad_timer_add(&nomad->timer, nomad->header.duration);
-			nomad->info.nr_frames++;
-			if (!frame_decoded) {
-				nomad->info.sample_rate = nomad->header.samplerate;
-				nomad->info.channels = MAD_NCHANNELS(&nomad->header);
-				nomad->info.layer = nomad->header.layer;
-				nomad->info.dual_channel = nomad->header.mode == MAD_MODE_DUAL_CHANNEL;
-				nomad->info.joint_stereo = nomad->header.mode == MAD_MODE_JOINT_STEREO;
-
-				nomad->frame.header = nomad->header;
-				if (mad_frame_decode(&nomad->frame, &nomad->stream) == -1) {
-					if (nomad->stream.error == MAD_ERROR_BUFLEN)
-						continue;
-					if (!MAD_RECOVERABLE(nomad->stream.error)) {
-						d_print("unrecoverable frame level error.\n");
-						return -1;
-					}
-					if (nomad->stream.error == MAD_ERROR_LOSTSYNC)
-						handle_lost_sync(nomad);
-					continue;
-				}
-				frame_decoded = 1;
-				xing_parse(nomad);
-
-#if defined(DEBUG_XING)
-				if (nomad->has_xing && (nomad->xing.flags & XING_FRAMES))
-					d_print("xing: frames: %d (xing)\n", nomad->xing.nr_frames);
-#endif
-
-				if (nomad->fast) {
-					nomad->info.avg_bitrate = -1;
-					nomad->info.vbr = -1;
-					if (nomad->has_xing && (nomad->xing.flags & XING_FRAMES)) {
-						nomad->info.nr_frames = nomad->xing.nr_frames;
-						mad_timer_multiply(&nomad->timer, nomad->info.nr_frames);
-					} else {
-						nomad->info.nr_frames = nomad->info.filesize /
-							(nomad->stream.next_frame - nomad->stream.this_frame);
-						mad_timer_multiply(&nomad->timer, nomad->info.nr_frames);
-					}
-					break;
-				}
-			} else {
-				if (old_bitrate != nomad->header.bitrate)
-					nomad->info.vbr = 1;
-			}
-			old_bitrate = nomad->header.bitrate;
-		} else {
-			if (!MAD_RECOVERABLE(nomad->stream.error) && nomad->stream.error != MAD_ERROR_BUFLEN) {
+		if (mad_frame_decode(&nomad->frame, &nomad->stream) == -1) {
+			if (nomad->stream.error == MAD_ERROR_BUFLEN)
+				continue;
+			if (!MAD_RECOVERABLE(nomad->stream.error)) {
 				d_print("unrecoverable frame level error.\n");
 				return -1;
 			}
 			if (nomad->stream.error == MAD_ERROR_LOSTSYNC)
 				handle_lost_sync(nomad);
+			continue;
 		}
+
+		build_seek_index(nomad);
+		bitrate_sum += header->bitrate;
+		nomad->info.nr_frames++;
+
+		if (nomad->info.nr_frames == 1) {
+			// first valid frame
+			nomad->info.sample_rate = header->samplerate;
+			nomad->info.channels = MAD_NCHANNELS(header);
+			nomad->info.layer = header->layer;
+			nomad->info.dual_channel = header->mode == MAD_MODE_DUAL_CHANNEL;
+			nomad->info.joint_stereo = header->mode == MAD_MODE_JOINT_STEREO;
+
+			xing_parse(nomad);
+
+			if (nomad->fast) {
+				calc_fast(nomad);
+				break;
+			}
+		} else {
+			if (old_bitrate != header->bitrate)
+				nomad->info.vbr = 1;
+		}
+		old_bitrate = header->bitrate;
 	}
 	if (nomad->info.nr_frames == 0) {
 		d_print("error: not an mp3 file!\n");
@@ -395,9 +386,11 @@ start:
 		goto start;
 	}
 	nomad->cur_frame++;
-	if (nomad->info.filesize > 0)
+	if (nomad->info.filesize > 0) {
 		build_seek_index(nomad);
-	mad_timer_add(&nomad->timer, nomad->frame.header.duration);
+	} else {
+		mad_timer_add(&nomad->timer, nomad->frame.header.duration);
+	}
 	mad_synth_frame(&nomad->synth, &nomad->frame);
 	return 0;
 }
@@ -407,7 +400,6 @@ static void init_mad(struct nomad *nomad)
 	mad_stream_init(&nomad->stream);
 	mad_frame_init(&nomad->frame);
 	mad_synth_init(&nomad->synth);
-	mad_header_init(&nomad->header);
 	mad_timer_reset(&nomad->timer);
 	nomad->cur_frame = 0;
 	nomad->i = -1;
@@ -419,7 +411,6 @@ static void free_mad(struct nomad *nomad)
 	mad_stream_finish(&nomad->stream);
 	mad_frame_finish(&nomad->frame);
 	mad_synth_finish(nomad->synth);
-	mad_header_finish(&nomad->header);
 }
 
 static int do_open(struct nomad *nomad, int fast)
@@ -443,8 +434,8 @@ static int do_open(struct nomad *nomad, int fast)
 		nomad->info.sample_rate = nomad->frame.header.samplerate;
 		nomad->info.channels = MAD_NCHANNELS(&nomad->frame.header);
 		nomad->info.layer = nomad->frame.header.layer;
-		nomad->info.dual_channel = nomad->header.mode == MAD_MODE_DUAL_CHANNEL;
-		nomad->info.joint_stereo = nomad->header.mode == MAD_MODE_JOINT_STEREO;
+		nomad->info.dual_channel = nomad->frame.header.mode == MAD_MODE_DUAL_CHANNEL;
+		nomad->info.joint_stereo = nomad->frame.header.mode == MAD_MODE_JOINT_STEREO;
 
 		/* unknown */
 		nomad->info.duration = -1.0;
@@ -625,9 +616,8 @@ int nomad_time_seek(struct nomad *nomad, double pos)
 		if (rc == 0)
 			return 0;
 
-		if (mad_header_decode(&nomad->header, &nomad->stream) == 0) {
+		if (mad_header_decode(&nomad->frame.header, &nomad->stream) == 0) {
 			build_seek_index(nomad);
-			mad_timer_add(&nomad->timer, nomad->header.duration);
 		} else {
 			if (!MAD_RECOVERABLE(nomad->stream.error) && nomad->stream.error != MAD_ERROR_BUFLEN) {
 				d_print("unrecoverable frame level error.\n");
