@@ -186,6 +186,7 @@ static const char * const valid_sort_keys[] = {
 	"date",
 	"genre",
 	"filename",
+	"albumartist",
 	NULL
 };
 
@@ -308,6 +309,24 @@ static void set_passwd(unsigned int id, const char *buf)
 	}
 }
 
+static void get_replaygain_preamp(unsigned int id, char *buf)
+{
+	sprintf(buf, "%f", replaygain_preamp);
+}
+
+static void set_replaygain_preamp(unsigned int id, const char *buf)
+{
+	double val;
+	char *end;
+
+	val = strtod(buf, &end);
+	if (end == buf) {
+		error_msg("floating point number expected (dB)");
+		return;
+	}
+	player_set_rg_preamp(val);
+}
+
 static void get_softvol_state(unsigned int id, char *buf)
 {
 	sprintf(buf, "%d %d", soft_vol_l, soft_vol_r);
@@ -515,6 +534,48 @@ static void toggle_repeat(unsigned int id)
 	update_statusline();
 }
 
+const char * const replaygain_names[] = {
+	"disabled", "track", "album", NULL
+};
+
+static void get_replaygain(unsigned int id, char *buf)
+{
+	strcpy(buf, replaygain_names[replaygain]);
+}
+
+static void set_replaygain(unsigned int id, const char *buf)
+{
+	int tmp;
+
+	if (!parse_enum(buf, 0, 2, replaygain_names, &tmp))
+		return;
+	player_set_rg(tmp);
+}
+
+static void toggle_replaygain(unsigned int id)
+{
+	player_set_rg((replaygain + 1) % 3);
+}
+
+static void get_replaygain_limit(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[replaygain_limit]);
+}
+
+static void set_replaygain_limit(unsigned int id, const char *buf)
+{
+	int tmp;
+
+	if (!parse_bool(buf, &tmp))
+		return;
+	player_set_rg_limit(tmp);
+}
+
+static void toggle_replaygain_limit(unsigned int id)
+{
+	player_set_rg_limit(replaygain_limit ^ 1);
+}
+
 static void get_show_hidden(unsigned int id, char *buf)
 {
 	strcpy(buf, bool_names[show_hidden]);
@@ -706,6 +767,9 @@ static const struct {
 	DT(play_library)
 	DT(play_sorted)
 	DT(repeat)
+	DT(replaygain)
+	DT(replaygain_limit)
+	DN(replaygain_preamp)
 	DT(show_hidden)
 	DT(show_remaining_time)
 	DT(set_term_title)
@@ -874,8 +938,6 @@ void options_load(void)
 	char filename[512];
 	int i;
 
-	display_errors = 1;
-
 	/* initialize those that can't be statically initialized */
 	for (i = 0; str_defaults[i].name; i++)
 		option_set(str_defaults[i].name, str_defaults[i].value);
@@ -886,7 +948,7 @@ void options_load(void)
 		const char *def = DATADIR "/cmus/rc";
 
 		if (errno != ENOENT)
-			warn_errno("loading %s", filename);
+			error_msg("loading %s: %s", filename, strerror(errno));
 
 		/* load defaults */
 		if (source_file(def) == -1)
@@ -897,7 +959,7 @@ void options_load(void)
 	snprintf(filename, sizeof(filename), "%s/rc", cmus_config_dir);
 	if (source_file(filename) == -1) {
 		if (errno != ENOENT)
-			warn_errno("loading %s", filename);
+			error_msg("loading %s: %s", filename, strerror(errno));
 	}
 }
 
