@@ -43,6 +43,7 @@ enum {
 	QUEUE_VIEW,
 	BROWSER_VIEW,
 	FILTERS_VIEW,
+	HELP_VIEW,
 	NR_VIEWS
 };
 
@@ -82,10 +83,12 @@ enum {
 
 extern char *output_plugin;
 extern char *status_display_program;
+extern char *server_password;
 extern int auto_reshuffle;
 extern int confirm_run;
 extern int show_hidden;
 extern int show_remaining_time;
+extern int set_term_title;
 extern int play_library;
 extern int repeat;
 extern int shuffle;
