@@ -33,6 +33,7 @@
 #include <string.h>
 #include <sys/time.h>
 #include <stdarg.h>
+#include <math.h>
 
 enum producer_status {
 	PS_UNLOADED,
@@ -49,7 +50,7 @@ enum consumer_status {
 
 struct player_info player_info = {
 	.mutex = CMUS_MUTEX_INITIALIZER,
-	.filename = { 0, },
+	.ti = NULL,
 	.metadata = { 0, },
 	.status = PLAYER_STATUS_STOPPED,
 	.pos = 0,
@@ -69,6 +70,10 @@ struct player_info player_info = {
 /* continue playing after track is finished? */
 int player_cont = 1;
 
+enum replaygain replaygain;
+int replaygain_limit = 1;
+double replaygain_preamp = 6.0;
+
 static const struct player_callbacks *player_cbs = NULL;
 
 static sample_format_t buffer_sf;
@@ -85,8 +90,11 @@ static int consumer_running = 1;
 static enum consumer_status consumer_status = CS_STOPPED;
 static unsigned int consumer_pos = 0;
 
-/* usually same as consumer_pos, sometimes less than consumer_pos */
-static unsigned int soft_vol_pos;
+/* for replay gain and soft vol
+ * usually same as consumer_pos, sometimes less than consumer_pos
+ */
+static unsigned int scale_pos;
+static double replaygain_scale = 1.0;
 
 /* locking {{{ */
 
@@ -114,7 +122,7 @@ static void reset_buffer(void)
 {
 	buffer_reset();
 	consumer_pos = 0;
-	soft_vol_pos = 0;
+	scale_pos = 0;
 }
 
 static void set_buffer_sf(sample_format_t sf)
@@ -154,32 +162,37 @@ static inline void scale_sample(signed short *buf, int i, int vol)
 	int sample = buf[i];
 
 	if (sample < 0) {
-		buf[i] = (sample * vol - SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+		sample = (sample * vol - SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+		if (sample < -32768)
+			sample = -32768;
 	} else {
-		buf[i] = (sample * vol + SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+		sample = (sample * vol + SOFT_VOL_SCALE / 2) / SOFT_VOL_SCALE;
+		if (sample > 32767)
+			sample = 32767;
 	}
+	buf[i] = sample;
 }
 
-static void soft_vol_scale(char *buffer, unsigned int *countp)
+static void scale_samples(char *buffer, unsigned int *countp)
 {
 	signed short *buf;
 	unsigned int count = *countp;
 	int ch, bits, l, r, i;
 
-	BUG_ON(soft_vol_pos < consumer_pos);
+	BUG_ON(scale_pos < consumer_pos);
 
-	if (consumer_pos != soft_vol_pos) {
-		unsigned int offs = soft_vol_pos - consumer_pos;
+	if (consumer_pos != scale_pos) {
+		unsigned int offs = scale_pos - consumer_pos;
 
 		if (offs >= count)
 			return;
 		buffer += offs;
 		count -= offs;
 	}
-	soft_vol_pos += count;
+	scale_pos += count;
 	buf = (signed short *)buffer;
 
-	if (soft_vol_l == 100 && soft_vol_r == 100)
+	if (replaygain_scale == 1.0 && soft_vol_l == 100 && soft_vol_r == 100)
 		return;
 
 	ch = sf_get_channels(buffer_sf);
@@ -194,34 +207,90 @@ static void soft_vol_scale(char *buffer, unsigned int *countp)
 	if (soft_vol_r != 100)
 		r = soft_vol_db[soft_vol_r];
 
+	l *= replaygain_scale;
+	r *= replaygain_scale;
+
 	for (i = 0; i < count / 4; i++) {
 		scale_sample(buf, i * 2, l);
 		scale_sample(buf, i * 2 + 1, r);
 	}
 }
 
+static int parse_double(const char *str, double *val)
+{
+	char *end;
+
+	*val = strtod(str, &end);
+	return str == end;
+}
+
+static void update_rg_scale(void)
+{
+	const char *g, *p;
+	double gain, peak, db, scale, limit;
+
+	replaygain_scale = 1.0;
+	if (!player_info.ti || !replaygain)
+		return;
+
+	if (replaygain == RG_TRACK) {
+		g = comments_get_val(player_info.ti->comments, "replaygain_track_gain");
+		p = comments_get_val(player_info.ti->comments, "replaygain_track_peak");
+	} else {
+		g = comments_get_val(player_info.ti->comments, "replaygain_album_gain");
+		p = comments_get_val(player_info.ti->comments, "replaygain_album_peak");
+	}
+
+	if (!g || !p) {
+		d_print("gain or peak not available\n");
+		return;
+	}
+	if (parse_double(g, &gain) || parse_double(p, &peak)) {
+		d_print("could not parse gain (%s) or peak (%s)\n", g, p);
+		return;
+	}
+	if (peak < 0.05) {
+		d_print("peak (%g) is too small\n", peak);
+		return;
+	}
+
+	db = replaygain_preamp + gain;
+
+	scale = pow(10.0, db / 20.0);
+	replaygain_scale = scale;
+	limit = 1.0 / peak;
+	if (replaygain_limit && replaygain_scale > limit)
+		replaygain_scale = limit;
+
+	d_print("gain = %f, peak = %f, db = %f, scale = %f, limit = %f, replaygain_scale = %f\n",
+			gain, peak, db, scale, limit, replaygain_scale);
+}
+
 static inline unsigned int buffer_second_size(void)
 {
 	return sf_get_second_size(buffer_sf);
 }
 
-static inline int get_next(char **filename)
+static inline int get_next(struct track_info **ti)
 {
-	return player_cbs->get_next(filename);
+	return player_cbs->get_next(ti);
 }
 
 /* updating player status {{{ */
 
-static inline void file_changed(void)
+static inline void file_changed(struct track_info *ti)
 {
 	player_info_lock();
-	if (producer_status == PS_UNLOADED) {
-		player_info.filename[0] = 0;
+	if (player_info.ti)
+		track_info_unref(player_info.ti);
+
+	player_info.ti = ti;
+	if (ti) {
+		d_print("file: %s\n", ti->filename);
 	} else {
-		strncpy(player_info.filename, ip_get_filename(ip), sizeof(player_info.filename));
-		player_info.filename[sizeof(player_info.filename) - 1] = 0;
+		d_print("unloaded\n");
 	}
-	d_print("%s\n", player_info.filename);
+	update_rg_scale();
 	player_info.metadata[0] = 0;
 	player_info.file_changed = 1;
 	player_info_unlock();
@@ -438,22 +507,23 @@ static void __prebuffer(void)
 static void __producer_play(void)
 {
 	if (producer_status == PS_UNLOADED) {
-		char *filename;
+		struct track_info *ti;
 
-		if (get_next(&filename) == 0) {
+		if (get_next(&ti) == 0) {
 			int rc;
 
-			ip = ip_new(filename);
+			ip = ip_new(ti->filename);
 			rc = ip_open(ip);
 			if (rc) {
-				player_ip_error(rc, "opening file `%s'", filename);
+				player_ip_error(rc, "opening file `%s'", ti->filename);
 				ip_delete(ip);
+				track_info_unref(ti);
+				file_changed(NULL);
 			} else {
 				ip_setup(ip);
 				producer_status = PS_PLAYING;
+				file_changed(ti);
 			}
-			free(filename);
-			file_changed();
 		}
 	} else if (producer_status == PS_PLAYING) {
 		if (ip_seek(ip, 0.0) == 0) {
@@ -503,12 +573,12 @@ static void __producer_pause(void)
 	}
 }
 
-static void __producer_set_file(const char *filename)
+static void __producer_set_file(struct track_info *ti)
 {
 	__producer_unload();
-	ip = ip_new(filename);
+	ip = ip_new(ti->filename);
 	producer_status = PS_STOPPED;
-	file_changed();
+	file_changed(ti);
 }
 
 /* setting producer status }}} */
@@ -594,7 +664,7 @@ static int change_sf(sample_format_t sf, int drop)
 
 static void __consumer_handle_eof(void)
 {
-	char *filename;
+	struct track_info *ti;
 
 	if (ip_is_remote(ip)) {
 		__producer_stop();
@@ -603,31 +673,31 @@ static void __consumer_handle_eof(void)
 		return;
 	}
 
-	if (get_next(&filename) == 0) {
+	if (get_next(&ti) == 0) {
 		__producer_unload();
-		ip = ip_new(filename);
+		ip = ip_new(ti->filename);
 		producer_status = PS_STOPPED;
 		/* PS_STOPPED, CS_PLAYING */
 		if (player_cont) {
 			__producer_play();
 			if (producer_status == PS_UNLOADED) {
 				__consumer_stop();
-				file_changed();
+				track_info_unref(ti);
+				file_changed(NULL);
 			} else {
 				/* PS_PLAYING */
-				file_changed();
+				file_changed(ti);
 				if (!change_sf(ip_get_sf(ip), 0))
 					__prebuffer();
 			}
 		} else {
 			__consumer_drain_and_stop();
-			file_changed();
+			file_changed(ti);
 		}
-		free(filename);
 	} else {
 		__producer_unload();
 		__consumer_drain_and_stop();
-		file_changed();
+		file_changed(NULL);
 	}
 	__player_status_changed();
 }
@@ -702,8 +772,8 @@ static void *consumer_loop(void *arg)
 			}
 			if (size > space)
 				size = space;
-			if (soft_vol)
-				soft_vol_scale(rpos, &size);
+			if (soft_vol || replaygain)
+				scale_samples(rpos, &size);
 			rc = op_write(rpos, size);
 			if (rc < 0) {
 				d_print("op_write returned %d %s\n", rc,
@@ -930,10 +1000,10 @@ void player_pause(void)
 	player_unlock();
 }
 
-void player_set_file(const char *filename)
+void player_set_file(struct track_info *ti)
 {
 	player_lock();
-	__producer_set_file(filename);
+	__producer_set_file(ti);
 	if (producer_status == PS_UNLOADED) {
 		__consumer_stop();
 		goto out;
@@ -955,10 +1025,10 @@ out:
 	player_unlock();
 }
 
-void player_play_file(const char *filename)
+void player_play_file(struct track_info *ti)
 {
 	player_lock();
-	__producer_set_file(filename);
+	__producer_set_file(ti);
 	if (producer_status == PS_UNLOADED) {
 		__consumer_stop();
 		goto out;
@@ -991,6 +1061,17 @@ out:
 void player_seek(double offset, int relative)
 {
 	player_lock();
+	if (consumer_status == CS_STOPPED) {
+		__producer_play();
+		if (producer_status == PS_PLAYING) {
+			__consumer_play();
+			if (consumer_status != CS_PLAYING) {
+				__producer_stop();
+				player_unlock();
+				return;
+			}
+		}
+	}
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED) {
 		double pos, duration, new_pos;
 		int rc;
@@ -1027,10 +1108,10 @@ void player_seek(double offset, int relative)
 				player_unlock();
 				return;
 			}
-			if (new_pos > duration) {
-				d_print("seek offset too large\n");
-				player_unlock();
-				return;
+			if (new_pos > duration - 5.0) {
+				new_pos = duration - 5.0;
+				if (new_pos < 0.0)
+					new_pos = 0.0;
 			}
 		}
 /* 		d_print("seeking %g/%g (%g from eof)\n", new_pos, duration, duration - new_pos); */
@@ -1040,7 +1121,7 @@ void player_seek(double offset, int relative)
 			op_drop();
 			reset_buffer();
 			consumer_pos = new_pos * buffer_second_size();
-			soft_vol_pos = consumer_pos;
+			scale_pos = consumer_pos;
 			__consumer_position_update();
 		} else {
 			d_print("error: ip_seek returned %d\n", rc);
@@ -1187,15 +1268,54 @@ void player_set_soft_vol(int soft)
 	int l, r;
 
 	consumer_lock();
-	/* don't mess with soft_vol_pos if soft_vol is already true */
-	if (!soft_vol)
-		soft_vol_pos = consumer_pos;
+	/* don't mess with scale_pos if soft_vol or replaygain is already enabled */
+	if (!soft_vol && !replaygain)
+		scale_pos = consumer_pos;
 	op_set_soft_vol(soft);
 	if (!op_get_volume(&l, &r))
 		volume_update(l, r);
 	consumer_unlock();
 }
 
+void player_set_rg(enum replaygain rg)
+{
+	player_lock();
+	/* don't mess with scale_pos if soft_vol or replaygain is already enabled */
+	if (!soft_vol && !replaygain)
+		scale_pos = consumer_pos;
+	replaygain = rg;
+
+	player_info_lock();
+	update_rg_scale();
+	player_info_unlock();
+
+	player_unlock();
+}
+
+void player_set_rg_limit(int limit)
+{
+	player_lock();
+	replaygain_limit = limit;
+
+	player_info_lock();
+	update_rg_scale();
+	player_info_unlock();
+
+	player_unlock();
+}
+
+void player_set_rg_preamp(double db)
+{
+	player_lock();
+	replaygain_preamp = db;
+
+	player_info_lock();
+	update_rg_scale();
+	player_info_unlock();
+
+	player_unlock();
+}
+
 int player_set_op_option(unsigned int id, const char *val)
 {
 	int rc;
