@@ -122,7 +122,6 @@ cc_cxx_common()
 
 	common_cf=
 	common_lf=
-	COMPAT_LIBS=
 
 	case `uname -s` in
 	*BSD)
@@ -158,27 +157,9 @@ cc_cxx_common()
 	SunOS)
 		common_cf="$common_cf -D__EXTENSIONS__ -I/usr/local/include"
 		common_lf="$common_lf -R/usr/local/lib -L/usr/local/lib"
-
-		# this is ugly but can be removed after Solaris is either
-		# dead or fixed
-
-		# connect() etc.
-		try_link -lsocket && COMPAT_LIBS="$COMPAT_LIBS -lsocket"
-
-		# gethostbyname()
-		try_link -lnsl && COMPAT_LIBS="$COMPAT_LIBS -lnsl"
-
-		# nanosleep()
-		if try_link -lrt
-		then
-			COMPAT_LIBS="$COMPAT_LIBS -lrt"
-		elif try_link -lposix4
-		then
-			COMPAT_LIBS="$COMPAT_LIBS -lposix4"
-		fi
 		;;
 	esac
-	makefile_vars SOFLAGS LDSOFLAGS LDDLFLAGS COMPAT_LIBS
+	makefile_vars SOFLAGS LDSOFLAGS LDDLFLAGS
 }
 
 # CC, LD, CFLAGS, LDFLAGS, SOFLAGS, LDSOFLAGS, LDDLFLAGS
@@ -486,8 +467,10 @@ int main(int argc, char *argv[])
 check_header()
 {
 	argc check_header $# 1
-	msg_checking "for header <$1>"
-	if try_compile "#include <$1>" "$@"
+	__header="$1"
+	shift
+	msg_checking "for header <$__header>"
+	if try_compile "#include <$__header>" "$@"
 	then
 		msg_result yes
 		return 0
