@@ -11,14 +11,7 @@
 struct simple_track {
 	struct list_head node;
 	struct track_info *info;
-
 	unsigned int marked : 1;
-
-	/* -1.. */
-	int disc : 15;
-
-	/* -1.. */
-	int num : 16;
 };
 
 struct shuffle_track {
@@ -71,10 +64,9 @@ struct simple_track *simple_list_get_next(struct list_head *head, struct simple_
 struct simple_track *simple_list_get_prev(struct list_head *head, struct simple_track *cur,
 		int (*filter)(const struct simple_track *));
 
-int simple_track_cmp(const struct list_head *a_head, const struct list_head *b_head, const char * const *keys);
 void sorted_list_add_track(struct list_head *head, struct simple_track *track, const char * const *keys);
 
-void shuffle_list_add_track(struct list_head *head, struct list_head *node, int nr);
+void list_add_rand(struct list_head *head, struct list_head *node, int nr);
 void reshuffle(struct list_head *head);
 
 int simple_list_for_each_marked(struct list_head *head,
