@@ -22,6 +22,7 @@
 #include "uchar.h"
 #include "misc.h"
 #include "xmalloc.h"
+#include "utils.h"
 #include "debug.h"
 
 #include <string.h>
@@ -88,13 +89,13 @@ int track_info_matches(struct track_info *ti, const char *text, unsigned int fla
 			if (flags & TI_MATCH_TITLE && title && u_strcasestr(title, word))
 				continue;
 		} else {
-			/* compare with filename (without path) */
-			char *filename = strrchr(ti->filename, '/');
+			/* compare with url or filename without path */
+			char *filename = ti->filename;
 
-			if (filename == NULL) {
-				filename = ti->filename;
-			} else {
-				filename++;
+			if (!is_url(filename)) {
+				char *slash = strrchr(ti->filename, '/');
+				if (slash)
+					filename = slash + 1;
 			}
 			if (u_strcasestr_filename(filename, word))
 				continue;
@@ -105,3 +106,63 @@ int track_info_matches(struct track_info *ti, const char *text, unsigned int fla
 	free_str_array(words);
 	return matched;
 }
+
+static int xstrcasecmp(const char *a, const char *b)
+{
+	if (a == NULL) {
+		if (b == NULL)
+			return 0;
+		return -1;
+	} else if (b == NULL) {
+		return 1;
+	}
+	return u_strcasecmp(a, b);
+}
+
+int track_info_cmp(const struct track_info *a, const struct track_info *b, const char * const *keys)
+{
+	int i, res = 0;
+
+	for (i = 0; keys[i]; i++) {
+		const char *key = keys[i];
+		const char *av, *bv;
+
+		/* numeric compare for tracknumber and discnumber */
+		if (strcmp(key, "tracknumber") == 0) {
+			res = comments_get_int(a->comments, key) -
+				comments_get_int(b->comments, key);
+			if (res)
+				break;
+			continue;
+		}
+		if (strcmp(key, "discnumber") == 0) {
+			res = comments_get_int(a->comments, key) -
+				comments_get_int(b->comments, key);
+			if (res)
+				break;
+			continue;
+		}
+		if (strcmp(key, "filename") == 0) {
+			/* NOTE: filenames are not necessarily UTF-8 */
+			res = strcasecmp(a->filename, b->filename);
+			if (res)
+				break;
+			continue;
+		}
+		if (strcmp(key, "albumartist") == 0) {
+			av = comments_get_albumartist(a->comments);
+			bv = comments_get_albumartist(b->comments);
+			res = xstrcasecmp(av, bv);
+			if (res)
+				break;
+			continue;
+		}
+
+		av = comments_get_val(a->comments, key);
+		bv = comments_get_val(b->comments, key);
+		res = xstrcasecmp(av, bv);
+		if (res)
+			break;
+	}
+	return res;
+}
