@@ -485,7 +485,7 @@ static void album_add_track(struct album *album, struct tree_track *track)
 		const struct simple_track *a = (const struct simple_track *)track;
 		const struct simple_track *b = (const struct simple_track *)to_tree_track(item);
 
-		if (simple_track_cmp(&a->node, &b->node, album_track_sort_keys) < 0)
+		if (track_info_cmp(a->info, b->info, album_track_sort_keys) < 0)
 			break;
 	}
 	/* add before item */
@@ -495,7 +495,8 @@ static void album_add_track(struct album *album, struct tree_track *track)
 void tree_add_track(struct tree_track *track)
 {
 	const struct track_info *ti = tree_track_info(track);
-	const char *album_name, *artist_name, *albumartist;
+	const char *album_name, *artist_name, *albumartist,
+		  *artist_sort, *albumartist_sort;
 	struct artist *artist;
 	struct album *album;
 	int date;
@@ -503,20 +504,26 @@ void tree_add_track(struct tree_track *track)
 	album_name = comments_get_val(ti->comments, "album");
 	artist_name = comments_get_val(ti->comments, "artist");
 	albumartist = comments_get_val(ti->comments, "albumartist");
+	artist_sort = comments_get_val(ti->comments, "artistsort");
+	albumartist_sort = comments_get_val(ti->comments, "albumartistsort");
 
 	if (is_url(ti->filename)) {
 		artist_name = "<Stream>";
 		album_name = "<Stream>";
 	}
+
 	if (artist_name == NULL)
 		artist_name = "<No Name>";
 	if (album_name == NULL)
 		album_name = "<No Name>";
 
-	if (albumartist) {
-		/* FIXME: user might want to create custom categories  */
-		artist_name = "<Compilations>";
-	} else {
+	if (albumartist_sort)
+		artist_name = albumartist_sort;
+	else if (albumartist)
+		artist_name = albumartist;
+	else if (artist_sort)
+		artist_name = artist_sort;
+	else {
 		const char *compilation = comments_get_val(ti->comments, "compilation");
 
 		if (compilation && (!strcasecmp(compilation, "1")
@@ -536,7 +543,7 @@ void tree_add_track(struct tree_track *track)
 			window_changed(lib_track_win);
 		}
 	} else if (artist) {
-		date = comments_get_int(ti->comments, "date");
+		date = comments_get_date(ti->comments, "date");
 		album = artist_add_album(artist, album_name, date);
 		album_add_track(album, track);
 
@@ -546,7 +553,7 @@ void tree_add_track(struct tree_track *track)
 			/* album is not selected => no need to update track_win */
 		}
 	} else {
-		date = comments_get_int(ti->comments, "date");
+		date = comments_get_date(ti->comments, "date");
 		artist = add_artist(artist_name);
 		album = artist_add_album(artist, album_name, date);
 		album_add_track(album, track);
