@@ -56,7 +56,7 @@
 #include <signal.h>
 #include <stdarg.h>
 
-#if defined(__sun__)
+#if defined(__sun__) || defined(__CYGWIN__)
 /* TIOCGWINSZ */
 #include <termios.h>
 #include <ncurses.h>
@@ -70,23 +70,16 @@ char *tgoto(const char *cap, int col, int row);
 
 /* globals. documented in ui_curses.h */
 
+int cmus_running = 1;
 int ui_initialized = 0;
 enum ui_input_mode input_mode = NORMAL_MODE;
 int cur_view = TREE_VIEW;
 struct searchable *searchable;
-
-/* display parse errors? (command line) */
-int display_errors = 0;
-
 char *lib_filename = NULL;
 char *pl_filename = NULL;
 
 /* ------------------------------------------------------------------------- */
 
-/* currently playing file */
-static struct track_info *cur_track_info = NULL;
-
-static int running = 1;
 static char *lib_autosave_filename;
 static char *pl_autosave_filename;
 
@@ -470,44 +463,6 @@ static inline void fopt_set_time(struct format_option *fopt, int value, int empt
 	fopt->empty = empty;
 }
 
-static void fill_track_fopts(struct tree_track *track)
-{
-	const char *filename;
-	const struct track_info *ti = tree_track_info(track);
-	int num, disc;
-
-	if (using_utf8) {
-		filename = ti->filename;
-	} else {
-		utf8_encode(ti->filename);
-		filename = conv_buffer;
-	}
-	disc = track->shuffle_track.simple_track.disc;
-	num = track->shuffle_track.simple_track.num;
-
-	fopt_set_str(&track_fopts[TF_ARTIST], track->album->artist->name);
-	fopt_set_str(&track_fopts[TF_ALBUM], track->album->name);
-	fopt_set_int(&track_fopts[TF_DISC], disc, disc == -1);
-	fopt_set_int(&track_fopts[TF_TRACK], num, num == -1);
-	fopt_set_str(&track_fopts[TF_TITLE], comments_get_val(ti->comments, "title"));
-	fopt_set_str(&track_fopts[TF_YEAR], comments_get_val(ti->comments, "date"));
-	fopt_set_str(&track_fopts[TF_GENRE], comments_get_val(ti->comments, "genre"));
-	fopt_set_time(&track_fopts[TF_DURATION], ti->duration, ti->duration == -1);
-	fopt_set_str(&track_fopts[TF_PATHFILE], filename);
-	if (is_url(ti->filename)) {
-		fopt_set_str(&track_fopts[TF_FILE], filename);
-	} else {
-		const char *f;
-
-		f = strrchr(filename, '/');
-		if (f) {
-			fopt_set_str(&track_fopts[TF_FILE], f + 1);
-		} else {
-			fopt_set_str(&track_fopts[TF_FILE], filename);
-		}
-	}
-}
-
 static void fill_track_fopts_track_info(struct track_info *info)
 {
 	char *filename;
@@ -563,7 +518,7 @@ static void print_track(struct window *win, int row, struct iter *iter)
 		cursor_y = 1 + row;
 	}
 
-	fill_track_fopts(track);
+	fill_track_fopts_track_info(tree_track_info(track));
 
 	if (track_info_has_tag(tree_track_info(track))) {
 		format_print(print_buffer, track_win_w, track_win_format, track_fopts);
@@ -687,13 +642,14 @@ static void print_help(struct window *win, int row, struct iter *iter)
 	struct iter sel;
 	int selected;
 	int pos;
+	int active = 1;
 	char buf[512];
 	const struct help_entry *e = iter_to_help_entry(iter);
 	const struct cmus_opt *opt;
 
 	window_get_sel(win, &sel);
 	selected = iters_equal(iter, &sel);
-	bkgdset(pairs[selected << 1]);
+	bkgdset(pairs[(active << 2) | (selected << 1)]);
 
 	if (selected) {
 		cursor_x = 0;
@@ -942,11 +898,11 @@ static void do_update_statusline(void)
 	fopt_set_str(&status_fopts[SF_SHUFFLE], shuffle_strs[shuffle]);
 	fopt_set_str(&status_fopts[SF_PLAYLISTMODE], aaa_mode_names[aaa_mode]);
 
-	if (cur_track_info)
-		duration = cur_track_info->duration;
-
 	player_info_lock();
 
+	if (player_info.ti)
+		duration = player_info.ti->duration;
+
 	if (volume_max == 0) {
 		vol_left = vol_right = vol = -1;
 	} else {
@@ -982,7 +938,7 @@ static void do_update_statusline(void)
 			strcat(format, "vol: %v ");
 		}
 	}
-	if (cur_track_info && is_url(cur_track_info->filename))
+	if (player_info.ti && is_url(player_info.ti->filename))
 		strcat(format, "buf: %b ");
 	strcat(format, "%=");
 	if (play_library) {
@@ -1161,19 +1117,19 @@ static void do_update_titleline(void)
 {
 	bkgdset(pairs[CURSED_TITLELINE]);
 	player_info_lock();
-	if (cur_track_info) {
+	if (player_info.ti) {
 		int i, use_alt_format = 0;
 		char *wtitle;
 
-		fill_track_fopts_track_info(cur_track_info);
-		if (is_url(cur_track_info->filename)) {
+		fill_track_fopts_track_info(player_info.ti);
+		if (is_url(player_info.ti->filename)) {
 			const char *title = get_stream_title();
 
 			if (title == NULL)
 				use_alt_format = 1;
 			fopt_set_str(&track_fopts[TF_TITLE], title);
 		} else {
-			use_alt_format = !track_info_has_tag(cur_track_info);
+			use_alt_format = !track_info_has_tag(player_info.ti);
 		}
 
 		if (use_alt_format) {
@@ -1330,14 +1286,13 @@ void error_msg(const char *format, ...)
 {
 	va_list ap;
 
-	if (!display_errors)
-		return;
-
 	strcpy(error_buf, "Error: ");
 	va_start(ap, format);
 	vsnprintf(error_buf + 7, sizeof(error_buf) - 7, format, ap);
 	va_end(ap);
 
+	d_print("%s\n", error_buf);
+
 	msg_is_error = 1;
 	error_count++;
 
@@ -1493,11 +1448,6 @@ void enter_search_backward_mode(void)
 	update_commandline();
 }
 
-void quit(void)
-{
-	running = 0;
-}
-
 void update_colors(void)
 {
 	int i;
@@ -1525,8 +1475,8 @@ static void clear_error(void)
 {
 	time_t t = time(NULL);
 
-	/* error msg is visible at least 3s */
-	if (t - error_time < 3)
+	/* prevent accidental clearing of error messages */
+	if (t - error_time < 2)
 		return;
 
 	if (error_buf[0]) {
@@ -1552,22 +1502,21 @@ static void spawn_status_program(void)
 	status = player_info.status;
 	if (status == 1)
 		stream_title = get_stream_title();
-	player_info_unlock();
 
 	i = 0;
 	argv[i++] = xstrdup(status_display_program);
 
 	argv[i++] = xstrdup("status");
 	argv[i++] = xstrdup(status_strs[status]);
-	if (cur_track_info) {
+	if (player_info.ti) {
 		static const char *keys[] = {
 			"artist", "album", "discnumber", "tracknumber", "title", "date", NULL
 		};
 		int j;
 
-		if (is_url(cur_track_info->filename)) {
+		if (is_url(player_info.ti->filename)) {
 			argv[i++] = xstrdup("url");
-			argv[i++] = xstrdup(cur_track_info->filename);
+			argv[i++] = xstrdup(player_info.ti->filename);
 			if (stream_title) {
 				argv[i++] = xstrdup("title");
 				argv[i++] = xstrdup(stream_title);
@@ -1576,23 +1525,25 @@ static void spawn_status_program(void)
 			char buf[32];
 
 			argv[i++] = xstrdup("file");
-			argv[i++] = xstrdup(cur_track_info->filename);
+			argv[i++] = xstrdup(player_info.ti->filename);
 			for (j = 0; keys[j]; j++) {
 				const char *key = keys[j];
 				const char *val;
 
-				val = comments_get_val(cur_track_info->comments, key);
+				val = comments_get_val(player_info.ti->comments, key);
 				if (val) {
 					argv[i++] = xstrdup(key);
 					argv[i++] = xstrdup(val);
 				}
 			}
-			snprintf(buf, sizeof(buf), "%d", cur_track_info->duration);
+			snprintf(buf, sizeof(buf), "%d", player_info.ti->duration);
 			argv[i++] = xstrdup("duration");
 			argv[i++] = xstrdup(buf);
 		}
 	}
 	argv[i++] = NULL;
+	player_info_unlock();
+
 	if (spawn(argv, &status) == -1)
 		error_msg("couldn't run `%s': %s", status_display_program, strerror(errno));
 	for (i = 0; argv[i]; i++)
@@ -1685,13 +1636,6 @@ static void update(void)
 		player_info.metadata_changed;
 
 	if (player_info.file_changed) {
-		if (cur_track_info)
-			track_info_unref(cur_track_info);
-		if (player_info.filename[0] == 0) {
-			cur_track_info = NULL;
-		} else {
-			cur_track_info = cmus_get_track_info(player_info.filename);
-		}
 		player_info.file_changed = 0;
 		needs_title_update = 1;
 		needs_status_update = 1;
@@ -1836,7 +1780,7 @@ static void main_loop(void)
 	int rc, fd_high;
 
 	fd_high = server_socket;
-	while (running) {
+	while (cmus_running) {
 		fd_set set;
 		struct timeval tv;
 
@@ -1865,16 +1809,9 @@ static void main_loop(void)
 			continue;
 		}
 
-		if (FD_ISSET(server_socket, &set)) {
-			/* no error msgs for cmus-remote */
-			display_errors = 0;
-
+		if (FD_ISSET(server_socket, &set))
 			server_serve();
-		}
 		if (FD_ISSET(0, &set)) {
-			/* diplay errors for interactive commands */
-			display_errors = 1;
-
 			if (using_utf8) {
 				u_getch();
 			} else {
@@ -1896,7 +1833,7 @@ static void main_loop(void)
 	}
 }
 
-static int get_next(char **filename)
+static int get_next(struct track_info **ti)
 {
 	struct track_info *info;
 
@@ -1914,8 +1851,7 @@ static int get_next(char **filename)
 	if (info == NULL)
 		return -1;
 
-	*filename = xstrdup(info->filename);
-	track_info_unref(info);
+	*ti = info;
 	return 0;
 }
 
@@ -1981,7 +1917,7 @@ static void init_curses(void)
 		 * xterm:            xterm, xterm-{,16,88,256}color
 		 */
 		if (!strcmp(term, "screen")) {
-			t_ts = "\033k";
+			t_ts = "\033_";
 			t_fs = "\033\\";
 		} else if (!strncmp(term, "xterm", 5) ||
 			   !strncmp(term, "rxvt", 4) ||
@@ -2085,6 +2021,7 @@ static const char *usage =
 "\n"
 "      --listen ADDR   listen on ADDR instead of ~/.cmus/socket\n"
 "                      ADDR is either a UNIX socket or host[:port]\n"
+"                      WARNING: using TCP/IP is insecure!\n"
 "      --plugins       list available plugins and exit\n"
 "      --help          display this help and exit\n"
 "      --version       " VERSION "\n"
