@@ -20,7 +20,7 @@
 #ifndef _XMALLOC_H
 #define _XMALLOC_H
 
-#include <compiler.h>
+#include "compiler.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -68,13 +68,6 @@ static inline char * __MALLOC xstrdup(const char *str)
 
 char * __MALLOC xstrndup(const char *str, size_t n);
 
-static inline char * __MALLOC xxstrdup(const char *str)
-{
-	if (str == NULL)
-		return NULL;
-	return xstrdup(str);
-}
-
 static inline void free_str_array(char **array)
 {
 	int i;
